function [ sample_datevec, rtc_datevecs ] = get_rtc_datevec_of_ctd_sample( p_profile, p_sample_index )
%GET_TIMESTAMP_OF_CTD_SAMPLE Summary of this function goes here
%   Detailed explanation goes here

%% Parameter check
if ~exist('p_sample_index', 'var')
    p_sample_index = 1;
end

if ~isstruct(p_profile) || ~isa(p_sample_index, 'double')
    disp([mfilename ': Please check your input parameter!']);
    return;
end

%% Initialize return variable
sample_datevec = NaN(1, 6);

%% Find CTD sample

ctd_profile = [];

if ~isempty(p_profile.PROFILE_DATA)
    p = nemo.profile.sort_profile_data(p_profile);
    ctd_profile = p.PROFILE_DATA;
    clear p
else
    return
end

software_datevec = ...
    datevec( ...
        datetime([ ...
            fliplr(p_profile.OVERALL_MISSION_INFORMATION.start_date), ...
            p_profile.OVERALL_MISSION_INFORMATION.start_time 0 ...
        ]) ...
        + seconds(p_profile.PROFILE_TECHNICAL_DATA.xmit_ascent_start_time) ...
);
software_time = software_datevec(4:6);
% get internal clock (RTC) @ 2000m
rtc_time = p_profile.PROFILE_DATA( ...
    p_profile.PROFILE_DATA(:, 7) == max(p_profile.PROFILE_DATA(:, 7)), ...
    2:4);
% if there are multiple pressure levels get the one with the
% earliest time
rtc_time = sortrows(rtc_time, [1:3]);
if isempty(rtc_time) || isempty(software_datevec) % prevent errors if empty
   return;
end
rtc_time = rtc_time(end, :);

rtc_datevec = datevec(datetime([software_datevec(1:3), rtc_time]));

if (any(rtc_time(1) == 21:23) && any(software_time(1) == 0:3))
    % RTC is more trustworthy, so reduce the software date by 1
    rtc_datevec = datevec(datetime([software_datevec(1:3), rtc_time]) - days(1));
elseif (any(rtc_time(1) == 0:3) && any(software_time(1) == 21:23))
    % same here, but now add 1 day
    rtc_datevec = datevec(datetime([software_datevec(1:3), rtc_time]) + days(1));
end
        
        
        
% get ctd, rtc time
rtc_times = p_profile.PROFILE_DATA(:, 2:4);
% add pseudo date to profile samples
pseudo_datenums = datenum([ ...
    zeros(size(ctd_profile, 1), 2), ...
    flipud((1:size(ctd_profile, 1))'), ...
    rtc_times ...
]);

diff_in_seconds = mod(flipud(diff(flipud(pseudo_datenums))), 1) * 86400;

rtc_datevecs = [rtc_datevec];
for i = 1:length(diff_in_seconds)
    rtc_datevecs = [ rtc_datevecs; datevec(datetime(rtc_datevecs(i, :)) + seconds(diff_in_seconds(end - i + 1)))];
end
rtc_datevecs = flipud(rtc_datevecs);

sample_datevec = rtc_datevecs(p_sample_index);


end

